// Code for sending MIDI signals to MFB drum synth when a digital input is on
// Test code only, not for release.
// Alex Allmont 2012

// TODO, schematic required, google AVR MIDI for how to wire up to MIDI plug.
// attiny2313 with 20MhZ crystal.  Inputs on PINB 0, 1, 2.  Outputs on Tx

#include <avr/io.h>
#include <stdlib.h>
#include <avr/interrupt.h>
#include <avr/sleep.h>
#include <avr/delay.h>

#define MIDI_BAUD 31250
#define UBRR_BAUD ((F_CPU / (MIDI_BAUD * 16UL)) - 1)

void midiSend(uint8_t data)
{
  while (!(UCSRA & (1 << UDRE)))
  {
    // Wait until USART is ready
  }

  UDR = data;
}

// Pulse on port B pin 3 to indicate midi has been sent
static uint32_t ontick = 0;

int soundDrum(int note)
{
  midiSend(0x90);
  midiSend(note);
  midiSend(0x7F);
  ontick = 100000; // TODO tidy up hack
}

int main(void)
{
  uint8_t inputs = 0;
  DDRB = 0b1000;

  // Midi USART. Set the baud to 31250, 8 data bits, one stop bit.
  UBRRH = UBRR_BAUD >> 8;
  UBRRL = UBRR_BAUD;
  UCSRB = _BV(TXEN);
  UCSRC = _BV(UCSZ1) | _BV(UCSZ0);

  for(;;)
  {
    inputs = PINB;
    if (inputs & _BV(0))
    {
    	soundDrum(36); // MFB Bass
    }
    else if (inputs & _BV(1))
    {
    	soundDrum(42); // MFB Snare
    }
    else if (inputs & _BV(2))
    {
    	soundDrum(38); // MFB Hat
    }

    // Pulse debug info on pin 3
    if (ontick)
    {
    	PORTB |= _BV(3);
    	--ontick;
    }
    else
    {
    	PORTB &= ~_BV(3);
    }
  }
}
